searchData={"items":[{"type":"module","title":"Rephex","doc":"```elixir\ndefmodule RephexPgWeb.State do\n  @initial_state %{\n    count: 0,\n  }\n\n  use Rephex.State, initial_state: @initial_state\n\n  def add_count(socket, %{amount: amount} = _payload) when is_integer(amount) do\n    # You can use `update_state`, `update_state_in` and `put_state_in` to update state\n    update_state_in(socket, [:count], &(&1 + amount))\n  end\nend\n```\n\n```elixir\ndefmodule RephexPgWeb.AccountLive.Index do\n  alias RephexPgWeb.State\n  use RephexPgWeb, :live_view\n  use Rephex.LiveView\n\n  alias Phoenix.LiveView.{AsyncResult, Socket}\n  alias RephexPgWeb.AccountLive.ComponentA\n\n  @impl true\n  def mount(_params, _session, %Socket{} = socket) do\n    {:ok, socket |> State.init()}\n  end\n\n  @impl true\n  def handle_event(\"add_count\", %{\"amount\" => amount}, %Socket{} = socket) do\n    {am, _} = Integer.parse(amount)\n\n    {:noreply, socket |> State.add_count(%{amount: am})}\n  end\n\n  @impl true\n  def render(assigns) do\n    # At default, Rephex state is assigned at `:rpx`.\n    # You can change root key by config.\n    ~H\"\"\"\n     Count: <%= @rpx.count %> \n     \n      [Add Count 1]\n     \n    <.live_component module={ComponentA} id=\"cmp_a\" rpx={@rpx} />\n    \"\"\"\n  end\nend\n```\n\n```elixir\ndefmodule RephexPgWeb.AccountLive.ComponentA do\n  use RephexPgWeb, :live_component\n  use Rephex.LiveComponent\n\n  alias Phoenix.LiveView.Socket\n\n  @initial_local_state %{}\n\n  @impl true\n  def mount(%Socket{} = socket) do\n    {:ok, socket |> assign(@initial_local_state)}\n  end\n\n  @impl true\n  def update(assigns, socket) do\n    {:ok,\n     socket\n     |> propagate_rephex(assigns)}\n  end\n\n  @impl true\n  def handle_event(\"add_count\", %{\"amount\" => amount}, %Socket{} = socket) do\n    {am, _} = Integer.parse(amount)\n\n    {:noreply,\n     socket\n     |> call_in_root(fn socket ->\n       State.add_count(socket, %{amount: am})\n     end)}\n  end\n\n  @impl true\n  def render(assigns) do\n    ~H\"\"\"\n     \n      [Add Count 2]\n     \n    \"\"\"\n  end\nend\n```","ref":"Rephex.html"},{"type":"function","title":"Rephex.root/0","doc":"Get root key of Rephex state. Default key is `:rpx`.\nRephex state will be contained at `socket.assigns[Rephex.root()]`.\n\nYou can change key by config.\n\nExample:\n\n    config :rephex, root: :my_rpx","ref":"Rephex.html#root/0"},{"type":"behaviour","title":"Rephex.Api.KernelApi","doc":"The behaviour for the Kernel sub-effect.","ref":"Rephex.Api.KernelApi.html"},{"type":"function","title":"Rephex.Api.KernelApi.send/2","doc":"","ref":"Rephex.Api.KernelApi.html#send/2"},{"type":"callback","title":"Rephex.Api.KernelApi.send/2","doc":"","ref":"Rephex.Api.KernelApi.html#c:send/2"},{"type":"module","title":"Rephex.Api.KernelApi.Impl","doc":"The implementation of the Kernel sub-effect.","ref":"Rephex.Api.KernelApi.Impl.html"},{"type":"function","title":"Rephex.Api.KernelApi.Impl.send/2","doc":"","ref":"Rephex.Api.KernelApi.Impl.html#send/2"},{"type":"behaviour","title":"Rephex.Api.LiveViewApi","doc":"The behaviour for the LiveView sub-effect.","ref":"Rephex.Api.LiveViewApi.html"},{"type":"function","title":"Rephex.Api.LiveViewApi.cancel_async/3","doc":"","ref":"Rephex.Api.LiveViewApi.html#cancel_async/3"},{"type":"callback","title":"Rephex.Api.LiveViewApi.cancel_async/3","doc":"","ref":"Rephex.Api.LiveViewApi.html#c:cancel_async/3"},{"type":"function","title":"Rephex.Api.LiveViewApi.start_async/3","doc":"","ref":"Rephex.Api.LiveViewApi.html#start_async/3"},{"type":"callback","title":"Rephex.Api.LiveViewApi.start_async/3","doc":"","ref":"Rephex.Api.LiveViewApi.html#c:start_async/3"},{"type":"module","title":"Rephex.Api.LiveViewApi.Impl","doc":"The implementation of the LiveView sub-effect.","ref":"Rephex.Api.LiveViewApi.Impl.html"},{"type":"function","title":"Rephex.Api.LiveViewApi.Impl.cancel_async/3","doc":"","ref":"Rephex.Api.LiveViewApi.Impl.html#cancel_async/3"},{"type":"function","title":"Rephex.Api.LiveViewApi.Impl.start_async/3","doc":"","ref":"Rephex.Api.LiveViewApi.Impl.html#start_async/3"},{"type":"behaviour","title":"Rephex.Api.SystemApi","doc":"The behaviour for the System sub-effect.","ref":"Rephex.Api.SystemApi.html"},{"type":"function","title":"Rephex.Api.SystemApi.monotonic_time/1","doc":"","ref":"Rephex.Api.SystemApi.html#monotonic_time/1"},{"type":"callback","title":"Rephex.Api.SystemApi.monotonic_time/1","doc":"","ref":"Rephex.Api.SystemApi.html#c:monotonic_time/1"},{"type":"module","title":"Rephex.Api.SystemApi.Impl","doc":"The implementation of the System sub-effect.","ref":"Rephex.Api.SystemApi.Impl.html"},{"type":"function","title":"Rephex.Api.SystemApi.Impl.monotonic_time/1","doc":"","ref":"Rephex.Api.SystemApi.Impl.html#monotonic_time/1"},{"type":"module","title":"Rephex.AsyncAction","doc":"Facilitates asynchronous operations in Phoenix LiveViews with enhanced state management.\n\n`Rephex.AsyncAction` seamlessly integrates with `Phoenix.LiveView` to manage asynchronous tasks,\nparticularly useful for operations that require real-time feedback to the user,\nsuch as loading data or performing long-running tasks.\n\n- When executing `start/3`, if `Phoenix.LiveView.AsyncResult` is not in a loading state, it changes the specified `AsyncResult` to a loading state before calling `Phoenix.LiveView.start_async`.\n- Within the `start_async/4` function, calling the `progress` function allows for changing the loading state of `AsyncResult`.\n- If `start_async/4` returns a value, `AsyncResult.ok` is called. In the case of an exception, `AsyncResult.failed` is called.","ref":"Rephex.AsyncAction.html"},{"type":"module","title":"Example: - Rephex.AsyncAction","doc":"# AsyncAction need Rephex state.\n    defmodule RephexPgWeb.State do\n      alias Phoenix.LiveView.AsyncResult\n\n      @initial_state %{\n        count: 0,\n        double_value: AsyncResult.ok(0),\n        add_twice_async: AsyncResult.ok(0)\n      }\n\n      use Rephex.State, initial_state: @initial_state\n\n      def add_count(socket, %{amount: amount} = _payload) when is_integer(amount) do\n        update_state_in(socket, [:count], &(&1 + amount))\n      end\n    end\n\n    # Minimal implementation\n    defmodule RephexPgWeb.State.HeavyDoubleAsync do\n      use Rephex.AsyncAction, result_path: [:double_value]\n\n      @impl true\n      def initial_progress(_path, _payload) do\n        # optional but recommended\n        # `start/4` apply this progress synchronously.\n        # AsyncResult.loading will be `{progress, _meta_values}` before start_async.\n        {0, 100}\n      end\n\n      @impl true\n      def start_async(_state, _path, %{amount: amount} = _payload, progress) do\n        # required\n        # This function will be passed to Phoenix's `start_async`.\n        max = 100\n        progress.({0, max})\n\n        1..max\n        |> Enum.each(fn i ->\n          :timer.sleep(2)\n          progress.({i, max})\n        end)\n\n        amount * 2\n        # AsyncAction will call `AsyncResult.ok(prev, amount)` on `handle_async`.\n      end\n    end\n\n    # Full implementation\n    defmodule RephexPgWeb.State.AddCountTwiceAsync do\n      alias Phoenix.LiveView\n      alias RephexPgWeb.State\n\n      @type payload :: %{amount: integer()}\n      @type progress :: {current :: non_neg_integer(), total :: non_neg_integer()}\n      @type cancel_reason :: any()\n\n      use Rephex.AsyncAction,\n        result_path: [:add_twice_async],\n        # You can pass types for functions implemented in macro.\n        payload_type: payload,\n        cancel_reason_type: cancel_reason,\n        progress_type: progress,\n        # You can suppress hyper frequent progress updates by setting throttle.\n        progress_throttle: 100\n\n      @impl true\n      def before_start(socket, _result_path, %{amount: _amount} = _payload) do\n        # optional\n        # This function will be called before `start_async`.\n        socket |> LiveView.put_flash(:info, \"Add twice start\")\n      end\n\n      @impl true\n      def initial_progress(_path, _payload) do\n        # optional\n        # This function will be called before `start_async` and determine the initial progress.\n        # AsyncResult.loading will be `{progress, _meta_values}` before start_async.\n        {0, 1}\n      end\n\n      @impl true\n      def start_async(_state, _path, %{amount: amount} = _payload, progress) do\n        # required\n        # This function will be passed to Phoenix's `start_async`.\n        max = 500\n        progress.({0, max})\n\n        1..max\n        |> Enum.each(fn i ->\n          :timer.sleep(2)\n          progress.({i, max})\n        end)\n\n        amount\n      end\n\n      @impl true\n      def after_resolve(socket, _result_path, result) do\n        # optional\n        # This function will be called after `start_async` is finished.\n        case result do\n          {:ok, amount} ->\n            socket\n            |> State.add_count(%{amount: amount})\n            |> LiveView.put_flash(:info, \"Add twice done: #{amount}\")\n\n          {:exit, _reason} ->\n            socket\n            |> LiveView.put_flash(:error, \"Add twice failed\")\n        end\n      end\n\n      @impl true\n      def generate_failed_value(_result_path, exit_reason) do\n        # optional\n        # You can customize the failed value.\n        case exit_reason do\n          {:shutdown, :cancel} -> \"canceled by no-reason\"\n          {:shutdown, {:cancel, text}} when is_bitstring(text) -> \"canceled by #{text}\"\n          _ -> \"unknown reason\"\n        end\n      end\n    end\n\n    # Usage in LiveView\n    defmodule RephexPgWeb.AccountLive.Index do\n      alias RephexPgWeb.State\n      use RephexPgWeb, :live_view\n      use Rephex.LiveView\n\n      alias Phoenix.LiveView.AsyncResult\n\n      @impl true\n      def mount(_params, _session, socket) do\n        {:ok, socket |> State.init()}\n      end\n\n      @impl true\n      def handle_event(\"start_heavy_double\", %{\"amount\" => amount}, socket) do\n        {am, _} = Integer.parse(amount)\n\n        {:noreply, socket |> State.HeavyDoubleAsync.start(%{amount: am})}\n      end\n\n      @impl true\n      def handle_event(\"cancel_heavy_double\", _params, socket) do\n        {:noreply, socket |> State.HeavyDoubleAsync.cancel()}\n      end\n\n      @impl true\n      def handle_event(\n            \"start_add_count_twice\",\n            %{\"force\" => force, \"amount\" => amount},\n            socket\n          )\n          when force in [\"1\", \"0\"] do\n        {am, _} = Integer.parse(amount)\n\n        {:noreply,\n        socket\n        |> State.AddCountTwiceAsync.start(%{amount: am}, restart_if_running: force == \"1\")}\n      end\n\n      @impl true\n      def handle_event(\"cancel_add_count_twice\", _params, socket) do\n        {:noreply, socket |> State.AddCountTwiceAsync.cancel({:shutdown, {:cancel, \"user cancel\"}})}\n      end\n\n      @impl true\n      def handle_event(\n            \"start_delayed_add\",\n            %{\"multi_key\" => key, \"amount\" => amount},\n            socket\n          ) do\n        {am, _} = Integer.parse(amount)\n        {:noreply, socket |> State.DelayedAddAsync.start(key, %{amount: am})}\n      end\n\n      def start_heavy_double_button(assigns) do\n        ~H\"\"\"\n         \n          <%= @text %>\n         \n        \"\"\"\n      end\n\n      def start_add_twice_button(assigns) do\n        ~H\"\"\"\n         \n          <%= @text %>\n         \n        \"\"\"\n      end\n\n      @impl true\n      def render(assigns) do\n        ~H\"\"\"\n         \n           Minimal AsyncAction Example \n           Double the amount with a delay. Async state is in AsyncResult. \n          <%= case @rpx.double_value do %>\n            <% %AsyncResult{loading: {{current, max}, _meta}} -> %>\n              <%= \"#{current} / #{max}\" %>\n            <% %AsyncResult{failed: reason} when reason != nil -> %>\n               Failed: <%= reason %> \n              <.start_heavy_double_button amount=\"2\" force=\"0\" text=\"Calculate double of 2\" />\n            <% %AsyncResult{ok?: true, result: result} -> %>\n              Double of amount: <%= result %>\n              <.start_heavy_double_button amount=\"2\" force=\"0\" text=\"Calculate double of 2\" />\n          <% end %>\n         \n\n         Count: <%= @rpx.count %> \n\n         \n           Full-implemented AsyncAction Example \n           We can manipulate values not in AsyncResult. \n          <%= case @rpx.add_twice_async do %>\n            <% %AsyncResult{loading: {{current, max}, _meta}} -> %>\n               \n                Cancel\n               \n              <.start_add_twice_button amount=\"2\" force=\"0\" text=\"Start without option\" />\n              <.start_add_twice_button amount=\"2\" force=\"1\" text=\"Force restart\" />\n              <%= \"#{current} / #{max}\" %>\n            <% %AsyncResult{failed: reason} when reason != nil -> %>\n              <.start_add_twice_button amount=\"2\" force=\"0\" text=\"Async add 2\" />\n               Failed: <%= reason %> \n            <% _ -> %>\n              <.start_add_twice_button amount=\"2\" force=\"0\" text=\"Async add 2\" />\n          <% end %>\n         \n        \"\"\"\n      end\n    end","ref":"Rephex.AsyncAction.html#module-example"},{"type":"module","title":"Rephex.AsyncAction.Backend","doc":"","ref":"Rephex.AsyncAction.Backend.html"},{"type":"function","title":"Rephex.AsyncAction.Backend.cancel/3","doc":"`YourAction.cancel(socket, ...)` call this.","ref":"Rephex.AsyncAction.Backend.html#cancel/3"},{"type":"function","title":"Rephex.AsyncAction.Backend.resolve/3","doc":"`LiveView.handle_async(..., result, socket)` call this.","ref":"Rephex.AsyncAction.Backend.html#resolve/3"},{"type":"function","title":"Rephex.AsyncAction.Backend.start/4","doc":"`YourAction.start(socket, ...)` call this.","ref":"Rephex.AsyncAction.Backend.html#start/4"},{"type":"function","title":"Rephex.AsyncAction.Backend.update_progress/3","doc":"`LiveView.handle_info(..., socket)` call this.","ref":"Rephex.AsyncAction.Backend.html#update_progress/3"},{"type":"type","title":"Rephex.AsyncAction.Backend.loading/0","doc":"","ref":"Rephex.AsyncAction.Backend.html#t:loading/0"},{"type":"behaviour","title":"Rephex.AsyncAction.Base","doc":"","ref":"Rephex.AsyncAction.Base.html"},{"type":"callback","title":"Rephex.AsyncAction.Base.after_resolve/3","doc":"After the async action is resolved, this callback will be called.","ref":"Rephex.AsyncAction.Base.html#c:after_resolve/3"},{"type":"callback","title":"Rephex.AsyncAction.Base.before_start/3","doc":"Before the async action starts, this callback will be called.","ref":"Rephex.AsyncAction.Base.html#c:before_start/3"},{"type":"callback","title":"Rephex.AsyncAction.Base.generate_failed_value/2","doc":"Generate failed value. This value will be set to AsyncResult via `AsyncResult.failed/2`.","ref":"Rephex.AsyncAction.Base.html#c:generate_failed_value/2"},{"type":"callback","title":"Rephex.AsyncAction.Base.initial_progress/2","doc":"Get initial progress. This value will be set synchronously before the async action starts.","ref":"Rephex.AsyncAction.Base.html#c:initial_progress/2"},{"type":"callback","title":"Rephex.AsyncAction.Base.options/0","doc":"This callback will be implemented by __using__.","ref":"Rephex.AsyncAction.Base.html#c:options/0"},{"type":"callback","title":"Rephex.AsyncAction.Base.start_async/4","doc":"Start async action.","ref":"Rephex.AsyncAction.Base.html#c:start_async/4"},{"type":"type","title":"Rephex.AsyncAction.Base.exit_reason/0","doc":"","ref":"Rephex.AsyncAction.Base.html#t:exit_reason/0"},{"type":"type","title":"Rephex.AsyncAction.Base.failed_value/0","doc":"","ref":"Rephex.AsyncAction.Base.html#t:failed_value/0"},{"type":"type","title":"Rephex.AsyncAction.Base.payload/0","doc":"","ref":"Rephex.AsyncAction.Base.html#t:payload/0"},{"type":"type","title":"Rephex.AsyncAction.Base.progress/0","doc":"","ref":"Rephex.AsyncAction.Base.html#t:progress/0"},{"type":"type","title":"Rephex.AsyncAction.Base.result_path/0","doc":"","ref":"Rephex.AsyncAction.Base.html#t:result_path/0"},{"type":"type","title":"Rephex.AsyncAction.Base.state/0","doc":"","ref":"Rephex.AsyncAction.Base.html#t:state/0"},{"type":"type","title":"Rephex.AsyncAction.Base.success_result/0","doc":"","ref":"Rephex.AsyncAction.Base.html#t:success_result/0"},{"type":"module","title":"Rephex.AsyncAction.Handler","doc":"","ref":"Rephex.AsyncAction.Handler.html"},{"type":"module","title":"Rephex.AsyncActionMulti","doc":"Manages multiple asynchronous operations under a specified map in Phoenix LiveViews,\nextending `Rephex.AsyncAction` capabilities.\n\n`Rephex.AsyncGroupAction` builds upon the foundation of `Rephex.AsyncAction`\nby introducing the management of multiple `AsyncResult` instances within a single, specified map.\n\nThis module is designed for scenarios where concurrent asynchronous tasks need to be\nexecuted and monitored within the same LiveView component,\noffering granular control over each task's lifecycle and state.","ref":"Rephex.AsyncActionMulti.html"},{"type":"module","title":"Example: - Rephex.AsyncActionMulti","doc":"# AsyncAction need Rephex state.\n    defmodule RephexPgWeb.State do\n      alias Phoenix.LiveView.AsyncResult\n\n      @initial_state %{\n        count: 0,\n        # AsyncActionMulti requires map will contain AsyncResult.\n        delayed_add_multi: %{}\n      }\n\n      use Rephex.State, initial_state: @initial_state\n\n      def add_count(socket, %{amount: amount} = _payload) when is_integer(amount) do\n        update_state_in(socket, [:count], &(&1 + amount))\n      end\n    end\n\n    # Minimal implementation\n    defmodule RephexPgWeb.State.DelayedAddAsync do\n      alias RephexPgWeb.State\n\n      use Rephex.AsyncActionMulti, result_map_path: [:delayed_add_multi]\n\n      @impl true\n      def start_async(_state, _path, %{amount: amount} = _payload, _progress) do\n        :timer.sleep(1000)\n        amount\n      end\n\n      @impl true\n      def after_resolve(socket, _result_path, result) do\n        case result do\n          {:ok, amount} ->\n            socket\n            |> State.add_count(%{amount: amount})\n\n          {:exit, _reason} ->\n            socket\n        end\n      end\n    end\n\n    # Usage in LiveView\n    defmodule RephexPgWeb.AccountLive.Index do\n      alias RephexPgWeb.State\n      use RephexPgWeb, :live_view\n      use Rephex.LiveView\n\n      alias Phoenix.LiveView.AsyncResult\n\n      @impl true\n      def mount(_params, _session, socket) do\n        {:ok, socket |> State.init()}\n      end\n\n      @impl true\n      def handle_event(\n            \"start_delayed_add\",\n            %{\"multi_key\" => key, \"amount\" => amount},\n            socket\n          ) do\n        {am, _} = Integer.parse(amount)\n        {:noreply, socket |> State.DelayedAddAsync.start(key, %{amount: am})}\n      end\n\n      def start_delayed_add_button(assigns) do\n        ~H\"\"\"\n         \n          <%= @text %>\n         \n        \"\"\"\n      end\n\n      @impl true\n      def render(assigns) do\n        ~H\"\"\"\n         \n           AsyncActionMulti Example \n           We can run multiple async actions with the same module. \n          <%= for i <- 1..3 do %>\n            <%= ~s{(AsyncResult is at [:delayed_add_multi, \"key-#{i}\"])} %>\n            <.start_delayed_add_button\n              amount={i}\n              multi_key={\"key-#{i}\"}\n              text={\"Start delayed add #{i} by key-#{i}\"}\n            />\n          <% end %>\n         \n        \"\"\"\n      end\n    end","ref":"Rephex.AsyncActionMulti.html#module-example"},{"type":"module","title":"Rephex.LiveComponent","doc":"Implement utility functions and handling functions for Rephex by `use Rephex.LiveComponent`.","ref":"Rephex.LiveComponent.html"},{"type":"module","title":"Example - Rephex.LiveComponent","doc":"defmodule ExampleWeb.AccountLive.ComponentA do\n      use ExampleWeb, :live_component\n      use Rephex.LiveComponent\n\n      @initial_local_state %{\n        count_double: 0\n      }\n\n      @impl true\n      def mount(%Socket{} = socket) do\n        {:ok, socket |> assign(@initial_local_state)}\n      end\n\n      @impl true\n      def update(assigns, socket) do\n        {:ok,\n        socket\n        |> propagate_rephex(assigns)\n        |> assign(:count_double, assigns.rpx.count * 2)}\n      end\n\n      def handle_event(\"event_in_component\", params, socket) do\n        # Mutate Rephex state in root LiveView\n        {:noreply,\n        socket\n        |> call_in_root(fn state -> mutation(state) end)}\n      end\n    end","ref":"Rephex.LiveComponent.html#module-example"},{"type":"function","title":"Rephex.LiveComponent.call_in_root/2","doc":"Call function in root LiveView.\n\nExample:\n\n    def handle_event(\"event_in_component\", params, socket) do\n      {:noreply,\n      socket\n      |> call_in_root(fn state -> mutation(state) end)}\n    end","ref":"Rephex.LiveComponent.html#call_in_root/2"},{"type":"function","title":"Rephex.LiveComponent.propagate_rephex/2","doc":"Assign rephex state to socket assigns in LiveComponent.\nWARN: Do NOT use this function in LiveView (root).\n\nExample:\n\n    def update(%{rpx: _} = assigns, socket) do\n      {:ok,\n      socket\n      |> propagate_rephex(assigns)\n      |> assign(other_state)}\n    end","ref":"Rephex.LiveComponent.html#propagate_rephex/2"},{"type":"module","title":"Rephex.LiveComponent.Handler","doc":"","ref":"Rephex.LiveComponent.Handler.html"},{"type":"function","title":"Rephex.LiveComponent.Handler.handle_info_by_call_in_root/2","doc":"","ref":"Rephex.LiveComponent.Handler.html#handle_info_by_call_in_root/2"},{"type":"module","title":"Rephex.LiveView","doc":"Implement handling functions for Rephex by `use Rephex.LiveView`.","ref":"Rephex.LiveView.html"},{"type":"module","title":"Example - Rephex.LiveView","doc":"defmodule ExampleWeb.AccountLive.Index do\n      use ExampleWeb, :live_view\n      use Rephex.LiveView\n\n      @impl true\n      def mount(_params, _session, socket) do\n        {:ok, socket |> ExampleWeb.State.init()}\n      end\n\n      @impl true\n      def render(assigns) do\n        ~H'''\n         Hello! \n        '''\n      end\n    end","ref":"Rephex.LiveView.html#module-example"},{"type":"module","title":"Rephex.State","doc":"Define Rephex state by `use`.\nDefined state must be initialized by `init/1` in `Phoenix.LiveView.mount/3`.","ref":"Rephex.State.html"},{"type":"module","title":"Example - Rephex.State","doc":"defmodule ExampleWeb.State do\n      @type t :: %{\n              count: integer(),\n              add_twice_async: %AsyncResult{}\n            }\n\n      @initial_state %{\n        count: 0,\n      }\n\n      use Rephex.State, initial_state: @initial_state\n\n      def add_count(socket, %{amount: amount} = _payload) when is_integer(amount) do\n        # You can use `update_state`, `update_state_in` and `put_state_in` to update state\n        update_state_in(socket, [:count], &(&1 + amount))\n      end\n    end\n\n    defmodule ExampleWeb.AccountLive.Index do\n      use ExampleWeb, :live_view\n      use Rephex.LiveView\n\n      @impl true\n      def mount(_params, _session, socket) do\n        {:ok, socket |> ExampleWeb.State.init()}\n      end\n    end","ref":"Rephex.State.html#module-example"},{"type":"function","title":"Rephex.State.init/2","doc":"","ref":"Rephex.State.html#init/2"},{"type":"module","title":"Rephex.State.Assigns","doc":"","ref":"Rephex.State.Assigns.html"},{"type":"function","title":"Rephex.State.Assigns.get_state/1","doc":"","ref":"Rephex.State.Assigns.html#get_state/1"},{"type":"function","title":"Rephex.State.Assigns.get_state_in/2","doc":"","ref":"Rephex.State.Assigns.html#get_state_in/2"},{"type":"function","title":"Rephex.State.Assigns.put_state_in/3","doc":"Update Rephex state by `put_in/3`.\n\nExample:\n\n    def put_value(socket, %{key: k, value: v} = _payload) do\n      put_state_in(socket, [:items, k], v)\n    end","ref":"Rephex.State.Assigns.html#put_state_in/3"},{"type":"function","title":"Rephex.State.Assigns.update_state/2","doc":"Update Rephex state.\n\nExample:\n    def add_count(socket, %{amount: amount} = _payload) do\n      update_state(socket, fn state -> %{state | count: state.count + amount} end)\n    end","ref":"Rephex.State.Assigns.html#update_state/2"},{"type":"function","title":"Rephex.State.Assigns.update_state_in/3","doc":"Update Rephex state by `update_in/3`.\n\nExample:\n```ex\nimport Rephex.State.Assigns\n\ndef mlt_count(socket, %{mlt: mlt} = _payload) do\n  update_state_in(socket, [:count], &(&1 * mlt))\nend\n```","ref":"Rephex.State.Assigns.html#update_state_in/3"},{"type":"module","title":"Rephex.Util","doc":"","ref":"Rephex.Util.html"},{"type":"function","title":"Rephex.Util.call_optional/3","doc":"```ex\ndefmodule BehaviourA do\n  @callback foo(any(), any()) :: any()\n\n  @optional_callbacks foo: 2\nend\n\ndefmodule BehaviourAImpl do\n  @behaviour BehaviourA\nend\n\ndefmodule User do\n  def call_foo() do\n    {:ok, :but_not_impl} =\n      Rephex.Util.call_optional(\n        {BehaviourAImpl, :foo, 2},\n        [1, 2],\n        {:ok, :but_not_impl}\n      )\n  end\nend\n```","ref":"Rephex.Util.html#call_optional/3"},{"type":"function","title":"Rephex.Util.socket_update_in/3","doc":"","ref":"Rephex.Util.html#socket_update_in/3"},{"type":"extras","title":"README","doc":"**Rephex**: Target to introduce the Power of [Redux-toolkit](https://redux-toolkit.js.org) to Phoenix LiveView.\n\n*Experimental: [README.md via ChatGPT is available](https://chat.openai.com/g/g-btova0uNt-rephex-readme-md)*\n\nBy integrating Rephex into your Phoenix LiveView projects, you unlock a suite of capabilities designed to enhance the structure, readability, and maintainability of your code:\n\n- **Decouple State Management from Views**: Achieve a clean separation between your application's state and its presentation layer, allowing for more manageable codebases and clearer state transitions.\n- **Child-Driven Assigns**: Empower smaller components to define their required assigns, streamlining data flow and component hierarchy for a more intuitive development experience.\n- **Clear State Demarcation**: Easily distinguish between global application states and the ephemeral local states of individual components, such as form inputs, enhancing the clarity of your component architecture.\n- **Simplified Asynchronous Operations**: Rephex simplifies the handling of asynchronous operations, making it easier to manage data fetching, processing, and more with minimal boilerplate.","ref":"readme.html"},{"type":"extras","title":"State Example - README","doc":"<!-- MODULEDOC -->\n\n```elixir\ndefmodule RephexPgWeb.State do\n  @initial_state %{\n    count: 0,\n  }\n\n  use Rephex.State, initial_state: @initial_state\n\n  def add_count(socket, %{amount: amount} = _payload) when is_integer(amount) do\n    # You can use `update_state`, `update_state_in` and `put_state_in` to update state\n    update_state_in(socket, [:count], &(&1 + amount))\n  end\nend\n```\n\n```elixir\ndefmodule RephexPgWeb.AccountLive.Index do\n  alias RephexPgWeb.State\n  use RephexPgWeb, :live_view\n  use Rephex.LiveView\n\n  alias Phoenix.LiveView.{AsyncResult, Socket}\n  alias RephexPgWeb.AccountLive.ComponentA\n\n  @impl true\n  def mount(_params, _session, %Socket{} = socket) do\n    {:ok, socket |> State.init()}\n  end\n\n  @impl true\n  def handle_event(\"add_count\", %{\"amount\" => amount}, %Socket{} = socket) do\n    {am, _} = Integer.parse(amount)\n\n    {:noreply, socket |> State.add_count(%{amount: am})}\n  end\n\n  @impl true\n  def render(assigns) do\n    # At default, Rephex state is assigned at `:rpx`.\n    # You can change root key by config.\n    ~H\"\"\"\n     Count: <%= @rpx.count %> \n     \n      [Add Count 1]\n     \n    <.live_component module={ComponentA} id=\"cmp_a\" rpx={@rpx} />\n    \"\"\"\n  end\nend\n```\n\n```elixir\ndefmodule RephexPgWeb.AccountLive.ComponentA do\n  use RephexPgWeb, :live_component\n  use Rephex.LiveComponent\n\n  alias Phoenix.LiveView.Socket\n\n  @initial_local_state %{}\n\n  @impl true\n  def mount(%Socket{} = socket) do\n    {:ok, socket |> assign(@initial_local_state)}\n  end\n\n  @impl true\n  def update(assigns, socket) do\n    {:ok,\n     socket\n     |> propagate_rephex(assigns)}\n  end\n\n  @impl true\n  def handle_event(\"add_count\", %{\"amount\" => amount}, %Socket{} = socket) do\n    {am, _} = Integer.parse(amount)\n\n    {:noreply,\n     socket\n     |> call_in_root(fn socket ->\n       State.add_count(socket, %{amount: am})\n     end)}\n  end\n\n  @impl true\n  def render(assigns) do\n    ~H\"\"\"\n     \n      [Add Count 2]\n     \n    \"\"\"\n  end\nend\n```\n\n<!-- MODULEDOC -->","ref":"readme.html#state-example"},{"type":"extras","title":"AsyncAction Example - README","doc":"```elixir\ndefmodule RephexPgWeb.State do\n  alias Phoenix.LiveView.AsyncResult\n\n  @initial_state %{\n    count: 0,\n    # AsyncAction requires AsyncResult.\n    # rpx.double_value.result: Result of AsyncAction.start_async\n    # rpx.double_value.loading: `{progress, _meta}` while AsyncAction is running. `progress/1` in `start_async/4` will update progress.\n    #   In this case, rpx.double_value.loading will be `{{current, max}, _meta}`\n    # rpx.double_value.failed: Result of AsyncAction.start_async (if canceled or exception raised)\n    double_value: AsyncResult.ok(0)\n  }\n\n  use Rephex.State, initial_state: @initial_state\n\n  def add_count(socket, %{amount: amount} = _payload) when is_integer(amount) do\n    # You can use `update_state`, `update_state_in` and `put_state_in` to update state\n    update_state_in(socket, [:count], &(&1 + amount))\n  end\nend\n```\n\n```elixir\ndefmodule RephexPgWeb.State.HeavyDoubleAsync do\n  use Rephex.AsyncAction, result_path: [:double_value]\n\n  @impl true\n  def initial_progress(_path, _payload) do\n    # optional but recommended\n    # `start/4` apply this progress synchronously.\n    # AsyncResult.loading will be `{progress, _meta_values}` before start_async.\n    {0, 100}\n  end\n\n  @impl true\n  def start_async(_state, _path, %{amount: amount} = _payload, progress) do\n    # required\n    # This function will be passed to Phoenix's `start_async`.\n    max = 100\n    progress.({0, max})\n\n    1..max\n    |> Enum.each(fn i ->\n      :timer.sleep(2)\n      # Update `rpx.double_value.loading` by AsyncResult.loading/2\n      progress.({i, max})\n    end)\n\n    amount * 2\n    # AsyncAction will call `AsyncResult.ok(prev, amount)` on `handle_event`.\n  end\nend\n```","ref":"readme.html#asyncaction-example"},{"type":"extras","title":"Installation - README","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `rephex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:rephex, \"~> 0.1.1\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#installation"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [0.1.1] - 2024-04-16\n\n#","ref":"changelog.html"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix Elixir dependency version. (~> 1.15 -> ~> 1.13)\n\n## [0.1.0] - 2024-04-16\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Rephex.State\n- Rephex.AsyncAction\n- Rephex.AsyncActionMulti\n\n[0.1.1]: https://github.com/sankaku-deltalab/rephex/releases/tag/0.1.0...v0.1.1\n[0.1.0]: https://github.com/sankaku-deltalab/rephex/releases/tag/0.1.0","ref":"changelog.html#added"}],"content_type":"text/markdown"}